module Sigmund
  module Liquid
    module Tags
      class ChildrenNav < ::Liquid::Tag

        Syntax = /(#{::Liquid::Expression}+)?/

        def initialize(tag_name, markup, tokens, context)
          if markup =~ Syntax
            begin
              @options = { id: 'nav', depth: 1, class: '', active_class: 'active'}
              markup.scan(::Liquid::TagAttributes) { |key, value| @options[key.to_sym] = value.gsub(/"|'/, '') }
              @options[:exclude] = Regexp.new(@options[:exclude]) if @options[:exclude]
              
              if @options[:snippet]
                template = @options[:snippet].include?('{') ? @options[:snippet] : context[:site].snippets.where(slug: @options[:snippet] ).try(:first).try(:template)
                @options[:liquid_render] = ::Liquid::Template.parse(template) unless template.blank?
              end
            rescue
              nil
            end
          else
            raise ::Liquid::SyntaxError.new("Syntax Error in 'children_nav ' - Valid syntax: children_nav <options>")
          end
          super
        end

        def render(context)          
          @site = context.registers[:site]
          @page = context.registers[:page] #current page for active class
          page = get_page_by_slug()
          render_entry_children(context, page, 1)
        end

        private

        # Determines root node for the list with the page slug
        def get_page_by_slug()
          return [] unless @options[:page_slug]
          page = @site.pages.where( slug: @options[:page_slug] ).first
          if page.nil?
            raise ::Liquid::SyntaxError.new("Slug '#{@options[:page_slug]}'' doesn't exists") 
          end
          page
        end

        # Returns a list element, a link to the page and its children
        def render_entry_link(context, page, css, depth, is_first, is_last, index )
          selected = @page.fullpath =~ /^#{page.fullpath}(\/.*)?$/ ? " #{@options[:active_class]}" : ''

          href = File.join('/', @site.localized_page_fullpath(page))
          title = render_title(context, page, href, selected, is_first, is_last, index)
          
          if @options[:liquid_render]
            return title
          end

          output  = %{<li id="#{page.slug.to_s.dasherize}-link" class="link#{selected} #{css}">}
          output << %{<a href="#{href}">#{title}</a>}
          output << render_entry_children(context, page, depth.succ) if (depth.succ <= @options[:depth].to_i)
          output << %{</li>}

          output.strip
        end

        # Recursively creates a nested unordered list for the depth specified
        def render_entry_children(context, page, depth)
          output = %{}
          children = page.children_with_minimal_attributes.reject { |c| !include_page?(c) }
          i = 0
          if children.present?
            children.each do |c, page|
              css = []
              css << 'first' if children.first == c
              css << 'last'  if children.last  == c
              i = i + 1
              output << '<ul>' if children.first == c
              output << render_entry_link(context, c, css.join(' '), depth, children.first == c, children.last  == c, i)
              output << '</ul>' if children.last == c
            end
          end
          output
        end

        def render_title(context, page, href, selected, is_first, is_last, index)
          if @options[:liquid_render]
            context.stack do
              context['page'] = page
              context['is_first'] = is_first
              context['is_last'] = is_last
              context['href'] = href
              context['selected'] = selected
              context['index'] = index
              @options[:liquid_render].render(context)
            end
          else
            page.title
          end
        end

        # Determines whether or not a page should be a part of the menu
        def include_page?(page)
          if !page.listed? || page.templatized? || !page.published?
            false
          elsif @options[:exclude]
            (page.fullpath =~ @options[:exclude]).nil?
          else
            true
          end
        end

      end

      ::Liquid::Template.register_tag('children_nav', ChildrenNav)
    end
  end
end
